# Script prepares firmware image for ViPNet SafeBoot installation (if needed)
# and writes it to SPI Flash BIOS region.
# Usage:
#  phase_fw.nsh <InstallPkgPartition> [<InstallProfile>]

echo -off

set -v InstallPkgPartition %1
set -v InstallProfile %2

set -v InstallPkgDir itinstall
set -v ToolsDir %InstallPkgDir%\tools
set -v BiosTool %ToolsDir%\BiosTool.efi
set -v FlasherWrapper %InstallPkgDir%\flasher.nsh
set -v BiosToolIni %InstallPkgDir%\modules\biostool.ini
set -v FfsModulesDir %InstallPkgDir%\modules\ffs
set -v PlatformDir %InstallPkgDir%\platform
set -v BiosToolLog %PlatformDir%\biostool.log
set -v BiosToolLog2 %PlatformDir%\biostool2.log
# base_fw_br.bin - base fw image to integrate modules to (case: vendor updates fw version and we
#                  need to use exactly this version to integrate modules to, but not current)
set -v BaseBiosRegion %PlatformDir%\base_fw_br.bin
# prep_fw_br.bin - prepared fw image, modules integration not needed (case: runtime integration
#                  on current platform is not supported)
set -v PrepBiosRegion %PlatformDir%\prep_fw_br.bin
# null_fw_br.bin - null fw image, fw update not needed (case: update modules at ESP only)
set -v NullBiosRegion %PlatformDir%\null_fw_br.bin
# cur_fw_br.bin  - original/current fw image to integrate modules to (readen from SPI flash)
set -v CurBiosRegion %PlatformDir%\cur_fw_br.bin
# mod_fw_br.bin  - modified fw image with integrated modules
set -v ModBiosRegion %PlatformDir%\mod_fw_br.bin

if x%InstallProfile% == x then
  set -v InstallProfile common_full
endif

#%InstallPkgPartition%:

if exist %NullBiosRegion% then
  echo 'Null' firware update mode
  # some temporary hack for this mode: copy all fs modules from install package to ESP
  set -v InstallProfile common_base
  exit /b 0x0
endif

if exist %CurBiosRegion% then
  rm -q %CurBiosRegion%
endif
if exist %ModBiosRegion% then
  rm -q %ModBiosRegion%
endif

# ExtNtfsDriverLoaded set in main script
if /i %ExtNtfsDriverLoaded% == false then
  set -v MinInstallProfile common_base
else
  set -v MinInstallProfile common_base_plus_extntfs
endif

# 0. read current fw image (for integration modules to or for backup)
%FlasherWrapper% read %CurBiosRegion% %ToolsDir%

if not %lasterror% == 0x0 then
  echo Reading current firmware image error (%lasterror%)
  goto Exit
endif
# workaround for BtFlash tool bug (not set lasterror in case of unsupported platform)
if not exist %CurBiosRegion% then
  echo Reading current firmware image error
  exit /b 0x2
endif

# 1. write prepared fw image if found
if exist %PrepBiosRegion% then
  echo 'Prepared image' firware update mode
  %FlasherWrapper% write %PrepBiosRegion% %ToolsDir%

  if not %lasterror% == 0x0 then
    echo Writing prepared firmware image error (%lasterror%)
  else
    # some temporary hack for this mode: copy all fs modules from install package to ESP
    set -v InstallProfile common_base
  endif

  goto Exit
endif

# 2. integrate ViPNet SafeBoot modules to fw image
echo UEFI modules integration in firmware image process may be so long (several minutes). Please wait...
if exist %InstallPkgPartition%:\%BaseBiosRegion% then
  echo 'New base image' firware update mode
  set -v InBiosRegion %BaseBiosRegion%
else
  echo 'Current base image' firware update mode
  set -v InBiosRegion %CurBiosRegion%
endif
echo Trying to apply install configuration: %InstallProfile%
cd %ToolsDir%
# absolute paths used below because of 'cd' command above:
# it needs for BiosTool correct call - install.dat file must be in current directory
%InstallPkgPartition%:\%BiosTool% -i %InstallPkgPartition%:\%InBiosRegion% -o %InstallPkgPartition%:\%ModBiosRegion% --profile %InstallProfile% --configfile %InstallPkgPartition%:\%BiosToolIni% --pmdzdir %InstallPkgPartition%:\%FfsModulesDir% --debug > %InstallPkgPartition%:\%BiosToolLog%

if not %lasterror% == 0x0 then
  if %InstallProfile% == common_full then
    set -v InstallProfile %MinInstallProfile%
    echo Trying to apply install configuration: %InstallProfile%
    %InstallPkgPartition%:\%BiosTool% -i %InstallPkgPartition%:\%InBiosRegion% -o %InstallPkgPartition%:\%ModBiosRegion% --profile %InstallProfile% --configfile %InstallPkgPartition%:\%BiosToolIni% --pmdzdir %InstallPkgPartition%:\%FfsModulesDir% --debug > %InstallPkgPartition%:\%BiosToolLog2%	
  else
    cd \
    echo Integrating ViPNet SafeBoot modules to firmware image error (%lasterror%)
	goto Exit
  endif
endif
cd \

# TODO?: type %BiosToolLog%

if not %lasterror% == 0x0 then
  echo Integrating ViPNet SafeBoot modules to firmware image error (%lasterror%)
  goto Exit
endif
# workaround for BiosTool tool bug (not set lasterror in case of insufficient DXE volume free space)
if not exist %InstallPkgPartition%:\%ModBiosRegion% then
  echo Integrating ViPNet SafeBoot modules to firmware image error
  exit /b 0x2
endif

echo ViPNet SafeBoot modules integrated successfully to firmware image
#echo Press 'q' key to cancel firmware update or other key to continue
#pause -q

# 3. write modified fw image
%FlasherWrapper% write %ModBiosRegion% %ToolsDir%

if not %lasterror% == 0x0 then
  echo Writing modified firmware image error (%lasterror%)
endif

:Exit
